unit Unit1;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Dialogs, Menus, Controls, StdCtrls, ComCtrls,
  ExtCtrls;

type TForm1 = class(TForm)
      ActionStart         : TButton;
      ActionStop          : TButton;
      RandomButton        : TButton;
      ExitButton          : TButton;

      IterationTrackBar   : TTrackBar;
      UpdateTrackBar      : TTrackBar;
      SeedTrackBar        : TTrackBar;
      BailTrackBar        : TTrackBar;

      ThresholdValueLabel : TLabel;
      UpdateValueLabel    : TLabel;
      SeedValueLabel      : TLabel;
      BailValueLabel      : TLabel;

      ThresholdLabel      : TLabel;
      UpdateLabel         : TLabel;
      SeedLabel           : TLabel;
      BailLabel           : TLabel;

      procedure QuitMessageQuery         (Sender : TObject; var Close : Boolean);
      procedure QuitQuery                (Sender : TObject);

      procedure InitForm1                (Sender : TObject);
      procedure DeInitForm1              (Sender : TObject);
      procedure PaintForm1               (Sender : TObject);

      procedure ActionStartClick         (Sender : TObject);
      procedure ActionStopClick          (Sender : TObject);
      procedure RandomButtonClick        (Sender : TObject);

      procedure IterationTrackBarChange  (Sender : TObject);
      procedure UpdateTrackBarChange     (Sender : TObject);
      procedure SeedTrackBarChange       (Sender : TObject);
      procedure BailTrackBarChange       (Sender : TObject);

      procedure ThresholdValueLabelClick (Sender : TObject);
      procedure UpdateValueLabelClick    (Sender : TObject);
      procedure SeedValueLabelClick      (Sender : TObject);
      procedure BailValueLabelClick      (Sender : TObject);

      private

      public

      function IterateBuddha (Cx,Cy : Double) : Boolean;
      procedure IteratedPlot (Cx,Cy,DeltaX,DeltaY : Double);
      procedure BuddhaPlot;
      procedure Render;
end;

var Form1          : TForm1;
    Bmp            : TBitmap;
    Density        : array[0..512,0..512] of Integer;
    DensityMax     : Integer = 1;
    SeedValue      : Integer = 256;
    BailValue      : Integer = 200;
    MaxIterations  : Integer = 4500;
    UpdateInterval : Integer = 12000;
    X1,X2,Y1,Y2    : Double;
    AllDone        : Boolean = False;
    sTime,eTime    : TDateTime;
    Exposed        : Longint;
implementation

{$R *.dfm}

{---------------Procedures to handle Exiting---------------}

procedure TForm1.QuitMessageQuery(Sender : TObject; var Close : Boolean);
begin
 AllDone := True;
 Halt(ord(AllDone));
end;

procedure TForm1.QuitQuery(Sender : TObject);
begin
 AllDone := True;
 Halt(ord(AllDone));
end;

{--------------Procedures to Init/DeInit stuff-------------}

procedure TForm1.InitForm1(Sender : TObject);
var I,J : Integer;
begin
 Bmp := Tbitmap.Create;
 Bmp.Width := Form1.ClientWidth;
 Bmp.Height := Form1.ClientHeight-(624-512);
 Form1.Caption := 'Buddhabrot v1.0b - [ Exposed Pixels: ' + IntToStr(Exposed)+' ] - [ Elapsed: ' + FormatDateTime('hh:mm:ss',(eTime-sTime))+' ]';
 Randseed := SeedValue;
 ThresholdValueLabel.Caption := IntToStr(MaxIterations);
 UpdateValueLabel.Caption := IntToStr(UpdateInterval);
 SeedValueLabel.Caption := IntToStr(SeedValue);
 BailValueLabel.Caption := IntToStr(BailValue);
 IterationTrackBar.Position := MaxIterations;
 UpdateTrackBar.Position := UpdateInterval;
 SeedTrackBar.Position := SeedValue;
 BailTrackBar.Position := BailValue;
 X1 := -2;
 Y1 := -1.5;
 X2 := 1;
 Y2 := 1.5;
 for I := 0 to Bmp.Width do for J := 0 to Bmp.Height do Bmp.Canvas.Pixels[I,J] := 0;
end;

procedure TForm1.DeInitForm1(Sender : TObject);
begin
 Bmp.Free;
end;

procedure TForm1.PaintForm1(Sender : TObject);
begin
 Self.Canvas.Draw(0,0,Bmp);
end;

procedure TForm1.RandomButtonClick(Sender : TObject);
begin
 SeedValue      := random(99999);
 BailValue      := random(99999);
 MaxIterations  := random(9999);
 UpdateInterval := random(99999);
 Form1.Caption := 'Buddhabrot v1.0b - [ Exposed Pixels: ' + IntToStr(Exposed)+' ] - [ Elapsed: ' + FormatDateTime('hh:mm:ss',(eTime-sTime))+' ]';
 Randseed := SeedValue;
 ThresholdValueLabel.Caption := IntToStr(MaxIterations);
 UpdateValueLabel.Caption := IntToStr(UpdateInterval);
 SeedValueLabel.Caption := IntToStr(SeedValue);
 BailValueLabel.Caption := IntToStr(BailValue);
 IterationTrackBar.Position := MaxIterations;
 UpdateTrackBar.Position := UpdateInterval;
 SeedTrackBar.Position := SeedValue;
 BailTrackBar.Position := BailValue;
end;

{---Functions/Procedures to iterate and plot the fractal---}

function TForm1.IterateBuddha(Cx,Cy : Double) : Boolean;
var Iterations     : Integer;
    X,Y,Xx,Yy,Temp : Extended;
begin
 X := Cx;
 Xx := X * X;
 Y := Cy;
 Yy := Y * Y;
 Iterations := 0;
 while (Iterations < MaxIterations) and ((Xx+Yy) < BailValue) do
  begin
   Temp := Cx + Xx - Yy;
   Y := Cy + 2 * X * Y;
   Yy := Y * Y;
   X := Temp;
   Xx := X * X;
   Inc(Iterations);
  end;
 Result:=True;
 if (Xx+Yy) >= BailValue then Result:=False;
end;

procedure TForm1.IteratedPlot(Cx,Cy,DeltaX,DeltaY : Double);
var iterations,Ix,Iy : Integer;
    X,Y,Xx,Yy,Temp   : Double;
begin
 X := Cx;
 Xx := X * X;
 Y := Cy;
 Yy := Y * Y;
 Iterations := 0;
 while (Iterations < MaxIterations) and ((Xx+Yy) < BailValue) do
  begin
   Temp := Cx + Xx - Yy;
   Y := Cy + 2 * X * Y;
   Yy := Y * Y;
   X := Temp;
   Xx := X * X;
   Ix := round((X-X1) / DeltaX);
   Iy := round((Y-Y1) / DeltaY);
   if (Ix >= 0) and (Ix <= Bmp.Width) and (Iy >= 0) and (Iy <= Bmp.Height) then
    begin
     Inc(Density[Iy,Ix]);
     if Density[iY,iX] > DensityMax then DensityMax := Density[Iy,Ix];
     Inc(Iterations);
    end;
  end;
 eTime := Time();
 Form1.Caption := 'Buddhabrot v1.0b - [ Exposed Pixels: ' + IntToStr(Exposed)+' ] - [ Elapsed: ' + FormatDateTime('hh:mm:ss',(eTime-sTime))+' ]';
end;

procedure TForm1.BuddhaPlot;
var Cx,Cy         : Double;
    DeltaX,DeltaY : Double;
    I,J           : Longint;
begin
 sTime := Time();
 DensityMax := 1;
 for I := 0 to Bmp.Width do for J := 0 to Bmp.Height do Density[I,J] := 0;
 DeltaX := (X2-X1) / Bmp.Width;
 DeltaY := (Y2-Y1) / Bmp.Height;
 I := 0;
 AllDone := False;
 while AllDone <> True do
  begin
    Cx := X1 + Random * (X2 - X1);
    Cy := Y1 + Random * (Y2 - Y1);
    if Iteratebuddha(Cx,Cy) <> True then
     begin
      IteratedPlot(Cx,Cy,DeltaX,DeltaY);
      if I mod UpdateInterval = 0 then
       begin
        eTime := Time();
        Form1.Caption := 'Buddhabrot v1.0b - [ Exposed Pixels: ' + IntToStr(Exposed)+' ] - [ Elapsed: ' + FormatDateTime('hh:mm:ss',(eTime-sTime))+' ]';
        Render;
       end;
      Inc(I);
      if I mod 256 = 0 then Exposed := I;
     end;
    Application.ProcessMessages;
  end;
 Render;
end;

procedure TForm1.Render;
var I,J       : Integer;
    ColorRamp : Double;
    Rgb       : Integer;
    {R,G,B     : integer;}
begin
 for I := 0 to Bmp.Width do
 for J := 0 to Bmp.Height do
  begin
   ColorRamp := Density[I,J] / (DensityMax / 2.5);
   if ColorRamp > 1 then ColorRamp := 1;
   {R := round( ColorRamp * (255+random(000)) );
   G := round( ColorRamp * (223+random(032)) );
   B := round( ColorRamp * (191+random(064)) );
   Bmp.Canvas.Pixels[I,J] := (R shl 16) or (G shl 8) or B;}
   Rgb := round( ColorRamp * 255);
   Bmp.Canvas.Pixels[I,J] := (Rgb shl 16) or (Rgb shl 8) or Rgb;
  end;
 Form1.Canvas.Draw(0,0,Bmp);
end;

{--------Procedures to handle buttons/sliders/labels--------}

procedure TForm1.ActionStartClick(Sender : TObject);
begin
 BuddhaPlot;
end;

procedure TForm1.ActionStopClick(Sender : TObject);
begin
 AllDone := True;
end;


procedure TForm1.IterationTrackBarChange(Sender : TObject);
begin
 MaxIterations := IterationTrackBar.Position;
 if MaxIterations < 00001 then MaxIterations := 00001;
 if MaxIterations > 99999 then MaxIterations := 99999;
 ThresholdValueLabel.Caption := IntToStr(MaxIterations);
end;

procedure TForm1.UpdateTrackBarChange(Sender : TObject);
begin
 UpdateInterval := UpdateTrackBar.Position;
 if UpdateInterval < 00001 then UpdateInterval := 00001;
 if UpdateInterval > 99999 then UpdateInterval := 99999;
 UpdateValueLabel.Caption := IntToStr(UpdateInterval);
end;

procedure TForm1.SeedTrackBarChange(Sender : TObject);
begin
 SeedValue := SeedTrackBar.Position;
 if SeedValue < 00001 then SeedValue := 00001;
 if SeedValue > 99999 then SeedValue := 99999;
 SeedValueLabel.Caption := IntToStr(SeedValue);
end;

procedure TForm1.BailTrackBarChange(Sender : TObject);
begin
 BailValue := BailTrackBar.Position;
 if BailValue < 00001 then BailValue := 00001;
 if BailValue > 99999 then BailValue := 99999;
 BailValueLabel.Caption := IntToStr(BailValue);
end;

procedure TForm1.ThresholdValueLabelClick(Sender: TObject);
begin
 MaxIterations := strtoint(inputbox('Input1','Threshold',IntToStr(MaxIterations)));
 if MaxIterations < 00001 then MaxIterations := 00001;
 if MaxIterations > 99999 then MaxIterations := 99999;
 ThresholdValueLabel.Caption := IntToStr(MaxIterations);
 IterationTrackBar.Position := MaxIterations;
end;

procedure TForm1.UpdateValueLabelClick(Sender: TObject);
begin
 UpdateInterval := strtoint(inputbox('Input2','Update Interval',IntToStr(UpdateInterval)));
 if UpdateInterval < 00001 then UpdateInterval := 00001;
 if UpdateInterval > 99999 then UpdateInterval := 99999;
 UpdateValueLabel.Caption := IntToStr(UpdateInterval);
 UpdateTrackBar.Position := UpdateInterval;
end;

procedure TForm1.SeedValueLabelClick(Sender: TObject);
begin
 SeedValue := strtoint(inputbox('Input3','Random Seed',IntToStr(SeedValue)));
 if SeedValue < 00001 then SeedValue := 00001;
 if SeedValue > 99999 then SeedValue := 99999;
 SeedValueLabel.Caption := IntToStr(SeedValue);
 SeedTrackBar.Position := SeedValue;
 Randseed:=SeedValue;
end;


procedure TForm1.BailValueLabelClick(Sender: TObject);
begin
 BailValue := StrToInt(inputbox('Input4','Bail Out Value',IntToStr(BailValue)));
 if BailValue < 00001 then BailValue := 00001;
 if BailValue > 99999 then BailValue := 99999;
 BailValueLabel.Caption := IntToStr(BailValue);
 BailTrackBar.Position := BailValue;
end;

end.
